% Script file: ErroreCS.m
%
% Analisi dell'errore per l'equazione del calore 
%            u_t = u_xx,  x in [0,L] 
% risolta con il metodo spettrale
% Lo script utilizza la funzione un1=CSPasso(un,dt,dx)
% Il dato iniziale e' 
%               u(x,0)=exp(-(x-L/2)^2)
% con condizioni al bordo periodiche u(0,t)=u(L,t)
%
close all;
clear; 
clc;
L = 10;
n0= 4;
N0 = 3 ;
Ne = 129 ;
xe=linspace(0,L,Ne);
dx0 = L/(N0-1);
t = n0*dx0 ;
kmax = 4 ;
passo = 8 ;
n = n0 ;
N = N0 ;
E=zeros(kmax,8*(N0-1)+1);

% Ripete kmax volte i calcoli con numero di punti crescente 

for k=1:kmax
   dx = L/(N-1);
   dt = dx;
   x=linspace(0,L,N);
   u=exp(-(x-L/2).^2);
   ue0=exp(-(xe-L/2).^2);
   plot(x,u,'o',xe,ue0,'r')
   xlabel('x')
   ylabel('u(x,t)')
   s=sprintf('Distribuzione iniziale per N=%1.0f:',N); 
   title(s);
   pause(1)
   tc = 0 ;
   
   % Calcola l'evoluzione al tempo t
   
   for h=0:n
      tc = tc+dt;
      u=CSPasso(u,dt,dx);
      ue=CSPasso(ue0,tc,L/(Ne-1));
      plot(x,real(u),'o',xe,real(ue),'r')
      s=sprintf('Evoluzione al tempo t=%f per N=%1.0f',tc,N);
      title(s);
      xlabel('x');
      ylabel('u(x,t)');
      axis([0 10 0 1]);
      pause(0.1)
   end
   ue=ue(1:8*passo:Ne);
   E(k,1:passo:8*(N0-1)+1)=abs(ue-u)/abs(ue);
   n=n*2;
   N=2*N-1;
   passo = passo/2;
end

% Realizza il grafico dell'errore

figure
passo=8;
N=N0;
hold on
for k=1:kmax
   x=linspace(0,L,N);
   Errore(k)=sum(E(k,1:passo:8*(N0-1)+1).^2)/N;
   plot(x,real(E(k,1:passo:8*(N0-1)+1)),'-o');
   passo = passo /2;
   N=2*N-1;
   s=sprintf('Errore al tempo finale t=%f',tc);
   title(s);
   xlabel('x');
   ylabel('E(x,t)');
end
hold off

% Tabella con errore massimo e ordine di convergenza

disp('EQUAZIONE DEL CALORE: metodo spettrale')
disp(' --------------------------------'); 
disp(sprintf('|N \t| Errore L2\t| Ordine\t|')); 
disp(' --------------------------------'); 
N=N0;
for k=1:kmax-1
   s=sprintf('|%1.0f\t| %3.2e\t|\t\t\t|',N,Errore(k));    
   disp(s);
   ordine(k)= log2(Errore(k)/Errore(k+1));
   s=sprintf('|\t|\t\t\t|\t%3.2f\t|',ordine(k));
   disp(s);
   N=2*N-1;
end
s=sprintf('|%1.0f\t| %3.2e\t|\t\t\t|',N,Errore(kmax));
disp(s);

